@@ -12,6 +12,9 @@
 #include "InOrderPieceSelector.h"
 #include "DownloadContext.h"
 #include "bittorrent_helper.h"
+#include "DiskAdaptor.h"
+#include "DiskWriterFactory.h"
+#include "PieceStatMan.h"
 
 namespace aria2 {
 
@@ -20,7 +23,9 @@ class DefaultPieceStorageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST_SUITE(DefaultPieceStorageTest);
   CPPUNIT_TEST(testGetTotalLength);
   CPPUNIT_TEST(testGetMissingPiece);
+  CPPUNIT_TEST(testGetMissingPiece_many);
   CPPUNIT_TEST(testGetMissingPiece_excludedIndexes);
+  CPPUNIT_TEST(testGetMissingPiece_manyWithExcludedIndexes);
   CPPUNIT_TEST(testGetMissingFastPiece);
   CPPUNIT_TEST(testGetMissingFastPiece_excludedIndexes);
   CPPUNIT_TEST(testHasMissingPiece);
@@ -31,21 +36,22 @@ class DefaultPieceStorageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testCancelPiece);
   CPPUNIT_TEST(testMarkPiecesDone);
   CPPUNIT_TEST(testGetCompletedLength);
+  CPPUNIT_TEST(testGetNextUsedIndex);
   CPPUNIT_TEST_SUITE_END();
 private:
-  SharedHandle<DownloadContext> _dctx;
+  SharedHandle<DownloadContext> dctx_;
   SharedHandle<Peer> peer;
   Option* option;
-  SharedHandle<PieceSelector> _pieceSelector;
+  SharedHandle<PieceSelector> pieceSelector_;
 public:
   void setUp() {
-    _dctx.reset(new DownloadContext());
-    bittorrent::load("test.torrent", _dctx);
+    dctx_.reset(new DownloadContext());
+    bittorrent::load("test.torrent", dctx_);
     peer.reset(new Peer("192.168.0.1", 6889));
-    peer->allocateSessionResource(_dctx->getPieceLength(),
-                                  _dctx->getTotalLength());
+    peer->allocateSessionResource(dctx_->getPieceLength(),
+                                  dctx_->getTotalLength());
     option = new Option();
-    _pieceSelector.reset(new InOrderPieceSelector());
+    pieceSelector_.reset(new InOrderPieceSelector());
   }
 
   void tearDown()
@@ -56,7 +62,9 @@ public:
 
   void testGetTotalLength();
   void testGetMissingPiece();
+  void testGetMissingPiece_many();
   void testGetMissingPiece_excludedIndexes();
+  void testGetMissingPiece_manyWithExcludedIndexes();
   void testGetMissingFastPiece();
   void testGetMissingFastPiece_excludedIndexes();
   void testHasMissingPiece();
@@ -67,22 +75,21 @@ public:
   void testCancelPiece();
   void testMarkPiecesDone();
   void testGetCompletedLength();
+  void testGetNextUsedIndex();
 };
 
 
 CPPUNIT_TEST_SUITE_REGISTRATION(DefaultPieceStorageTest);
 
 void DefaultPieceStorageTest::testGetTotalLength() {
-  DefaultPieceStorage pss(_dctx, option);
+  DefaultPieceStorage pss(dctx_, option);
 
   CPPUNIT_ASSERT_EQUAL((uint64_t)384ULL, pss.getTotalLength());
 }
 
 void DefaultPieceStorageTest::testGetMissingPiece() {
-  DefaultPieceStorage pss(_dctx, option);
-  pss.setPieceSelector(_pieceSelector);
-  pss.setEndGamePieceNum(0);
-
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
   peer->setAllBitfield();
 
   SharedHandle<Piece> piece = pss.getMissingPiece(peer);
@@ -98,10 +105,28 @@ void DefaultPieceStorageTest::testGetMissingPiece() {
   CPPUNIT_ASSERT(piece.isNull());
 }
 
+void DefaultPieceStorageTest::testGetMissingPiece_many() {
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
+  peer->setAllBitfield();
+  std::vector<SharedHandle<Piece> > pieces;
+  pss.getMissingPiece(pieces, 2, peer);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pieces.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
+                       pieces[0]->toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=1, length=128"),
+                       pieces[1]->toString());
+  pieces.clear();
+  pss.getMissingPiece(pieces, 2, peer);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, pieces.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
+                       pieces[0]->toString());
+}
+
 void DefaultPieceStorageTest::testGetMissingPiece_excludedIndexes()
 {
-  DefaultPieceStorage pss(_dctx, option);
-  pss.setPieceSelector(_pieceSelector);
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
   peer->setAllBitfield();
@@ -121,9 +146,27 @@ void DefaultPieceStorageTest::testGetMissingPiece_excludedIndexes()
   CPPUNIT_ASSERT(piece.isNull());
 }
 
+void DefaultPieceStorageTest::testGetMissingPiece_manyWithExcludedIndexes() {
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
+  peer->setAllBitfield();
+  std::vector<size_t> excludedIndexes;
+  excludedIndexes.push_back(1);
+  std::vector<SharedHandle<Piece> > pieces;
+  pss.getMissingPiece(pieces, 2, peer, excludedIndexes);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pieces.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=0, length=128"),
+                       pieces[0]->toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece: index=2, length=128"),
+                       pieces[1]->toString());
+  pieces.clear();
+  pss.getMissingPiece(pieces, 2, peer, excludedIndexes);
+  CPPUNIT_ASSERT(pieces.empty());
+}
+
 void DefaultPieceStorageTest::testGetMissingFastPiece() {
-  DefaultPieceStorage pss(_dctx, option);
-  pss.setPieceSelector(_pieceSelector);
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
   peer->setAllBitfield();
@@ -139,8 +182,8 @@ void DefaultPieceStorageTest::testGetMissingFastPiece() {
 
 void DefaultPieceStorageTest::testGetMissingFastPiece_excludedIndexes()
 {
-  DefaultPieceStorage pss(_dctx, option);
-  pss.setPieceSelector(_pieceSelector);
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
   peer->setAllBitfield();
@@ -159,7 +202,7 @@ void DefaultPieceStorageTest::testGetMissingFastPiece_excludedIndexes()
 }
 
 void DefaultPieceStorageTest::testHasMissingPiece() {
-  DefaultPieceStorage pss(_dctx, option);
+  DefaultPieceStorage pss(dctx_, option);
 
   CPPUNIT_ASSERT(!pss.hasMissingPiece(peer));
   
@@ -169,8 +212,8 @@ void DefaultPieceStorageTest::testHasMissingPiece() {
 }
 
 void DefaultPieceStorageTest::testCompletePiece() {
-  DefaultPieceStorage pss(_dctx, option);
-  pss.setPieceSelector(_pieceSelector);
+  DefaultPieceStorage pss(dctx_, option);
+  pss.setPieceSelector(pieceSelector_);
   pss.setEndGamePieceNum(0);
 
   peer->setAllBitfield();
@@ -191,7 +234,7 @@ void DefaultPieceStorageTest::testCompletePiece() {
 }
 
 void DefaultPieceStorageTest::testGetPiece() {
-  DefaultPieceStorage pss(_dctx, option);
+  DefaultPieceStorage pss(dctx_, option);
   
   SharedHandle<Piece> pieceGot = pss.getPiece(0);
   CPPUNIT_ASSERT_EQUAL((size_t)0, pieceGot->getIndex());
@@ -200,7 +243,7 @@ void DefaultPieceStorageTest::testGetPiece() {
 }
 
 void DefaultPieceStorageTest::testGetPieceInUsedPieces() {
-  DefaultPieceStorage pss(_dctx, option);
+  DefaultPieceStorage pss(dctx_, option);
   SharedHandle<Piece> piece = SharedHandle<Piece>(new Piece(0, 128));
   piece->completeBlock(0);
   pss.addUsedPiece(piece);
@@ -211,7 +254,7 @@ void DefaultPieceStorageTest::testGetPieceInUsedPieces() {
 }
 
 void DefaultPieceStorageTest::testGetPieceCompletedPiece() {
-  DefaultPieceStorage pss(_dctx, option);
+  DefaultPieceStorage pss(dctx_, option);
   SharedHandle<Piece> piece = SharedHandle<Piece>(new Piece(0, 128));
   pss.completePiece(piece);
   SharedHandle<Piece> pieceGot = pss.getPiece(0);
@@ -302,4 +345,16 @@ void DefaultPieceStorageTest::testGetCompletedLength()
   CPPUNIT_ASSERT_EQUAL((uint64_t)256*1024*1024, ps.getCompletedLength());
 }
 
+void DefaultPieceStorageTest::testGetNextUsedIndex()
+{
+  DefaultPieceStorage pss(dctx_, option);
+  CPPUNIT_ASSERT_EQUAL((size_t)3, pss.getNextUsedIndex(0));
+  SharedHandle<Piece> piece = pss.getMissingPiece(2);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pss.getNextUsedIndex(0));
+  pss.completePiece(piece);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pss.getNextUsedIndex(0));
+  piece = pss.getMissingPiece(0);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pss.getNextUsedIndex(0));
+}
+
 } // namespace aria2
